-- have some variables
local slomo = false
local explosions, debris, trail, smoke = {}, {}, {}, {}
local sounds = {}



function loadExplosions()
	math.randomseed(os.time())
	graphics={e=love.graphics.newImage("e.png"),d=love.graphics.newImage("d.png")}
	love.graphics.setBlendMode("additive")
	for i=1,8 do
		sounds[i] = love.audio.newSource("e.mp3", "static")
	end
end



function love.keypressed(k)
	if k == "escape" then love.event.push("q")
	else
		slomo = not slomo
		-- fix sound pitches
		for k,v in ipairs(sounds) do
			if slomo then v:setPitch(v:getPitch()/10)
			else v:setPitch(v:getPitch()*10)
			end
		end
	end
end



function updateExplosions(dt)
	dt = slomo and dt/10 or dt
	local dtf = 1-dt	-- velocity falloff factor
	
	local function tick(e, t,k)
		e.life = e.life - dt
		if e.life < 0 then table.remove(t,k) end
		e.x,	e.y,	e.r,	e.s  =	e.x+e.xv*dt,	e.y+e.yv*dt,	e.r+e.rv*dt,	e.s+e.sv*dt
		e.xv,	e.yv,	e.rv,	e.sv =	e.xv*dtf,		e.yv*dtf,		e.rv*dtf,		e.sv*dtf
	end
	
	for k,v in ipairs(explosions) do tick(v, explosions,k) end
	for k,v in ipairs(debris) do
		-- create a trail after it
		table.insert(trail, {
			x=v.x, y=v.y, r=v.r, s=v.s,
			xv=v.xv/5, yv=v.yv/5, rv=0, sv=v.sv,
			life=v.life/2
		})
		
		tick(v, debris,k)
		v.yv = v.yv + dt*1000	-- gravity
	end
	for k,v in ipairs(trail) do tick(v, trail,k) end
	for k,v in ipairs(smoke) do tick(v, smoke,k) end
end



function drawExplosions()
	local lg = love.graphics
	lg.setColor(255,230,150, math.sin(love.timer.getTime()*5)*64+191)
	lg.print("Try pressing Space! (Slow-motion doesn't violate the time limit, right?)\n  - Taehl (SelfMadeSpirit@gmail.com)", 9,9)
	
	for k,v in ipairs(explosions) do
		lg.setColor(255,230,150, v.life*1*255)
		lg.draw(graphics.e, v.x,v.y,v.r, v.s,v.s/v.life/10, 32,32)
	end
	
	for k,v in ipairs(debris) do
		lg.setColor(255,255,255, math.min(v.life*3*255, 255))
		lg.draw(graphics.e, v.x,v.y,v.r, v.s,v.s, 32,32)
	end
	
	for k,v in ipairs(trail) do
		lg.setColor(255,255,255, v.life*2*127)
		lg.draw(graphics.d, v.x,v.y,v.r, v.s,v.s, 16,32)
	end
	
	for k,v in ipairs(smoke) do
		lg.setColor(8,8,9, v.life/5*255)
		lg.draw(graphics.e, v.x,v.y,v.r, v.s,v.s, 32,32)
	end
end



function newExplosion(x,y)
	-- shockwave
	table.insert(explosions, {
		x=x, y=y, r=math.random()*6.3, s=0,
		xv=0, yv=-100, rv=math.random()*20-10, sv=80,
		life=math.random()/5+.2
	})
	
	-- cloud
	for i=1,math.random(50)+150 do
		local a,d = math.random()*6.3, math.random()*2000-1000 -- circular
		table.insert(explosions, {
			x=x, y=y, r=math.random()*6.3, s=math.random()*2+.2,
			xv=math.sin(a)*d, yv=math.cos(a)*d-500, rv=math.random()*20-10, sv=math.random()*10-5,
			life=math.random()/5+.2
		})
	end
	
	-- debris
	for i=1,math.random(50)+10 do
		local a,d = math.random()*6.3, math.random()*2000
		table.insert(debris, {
			x=x, y=y, r=a, s=math.random()/4+.1,
			xv=math.sin(a)*d, yv=-math.cos(a)*d, rv=math.random()*20-10, sv=math.random()*2-1,
			life=math.random()
		})
	end
	
	-- smoke
	for i=1,math.random(50)+50 do
		table.insert(smoke, {
			x=x,y=y, r=math.random()*6.3, s=1,
			xv=math.random()*800-400, yv=math.random()*-900, rv=math.random()*8-4, sv=math.random()*6,
			life=math.random()*5
		})
	end
	
	-- sound effect!
	for i=1,8 do
		sounds[i]:play()
		sounds[i]:setPitch(math.random()*.1+i/10 / (slomo and 10 or 1))
	end
end